"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ticketsRouter = exports.initializeTickets = exports.ticketCache = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const Config_1 = require("../Config");
const Logger = require("../Logger");
const ticketVerification_1 = require("../services/ticketVerification");
const errors_1 = require("../types/errors");
const ticketFilePath = (0, path_1.join)(__dirname, '..', '..', 'static', 'tickets.json');
// Export for testing
exports.ticketCache = null;
const CACHE_TTL = 60 * 1000; // 1 minute in milliseconds
const verificationConfig = {
    allowedTicketSigners: Config_1.config.tickets.allowedTicketSigners,
    minSigRequired: Config_1.config.tickets.minSigRequired,
    requiredSecurityLevel: Config_1.config.tickets.requiredSecurityLevel
};
function createApiError(code, message, details) {
    let statusCode = 500;
    // Map error codes to appropriate HTTP status codes
    if (code.startsWith('INVALID')) {
        statusCode = 400;
    }
    else if (code === 'TICKET_NOT_FOUND') {
        statusCode = 404;
    }
    return {
        statusCode,
        response: {
            error: message,
            code: errors_1.ErrorCodes[code],
            ...(details && { details })
        }
    };
}
function handleFileError(err) {
    Logger.mainLogger.error('Failed to read tickets file:', err);
    return createApiError('TICKETS_FILE_NOT_ACCESSIBLE', `Unable to access tickets configuration: ${ticketFilePath}`);
}
function handleJsonParseError(err) {
    Logger.mainLogger.error('Failed to parse tickets JSON:', err);
    return createApiError('INVALID_TICKETS_DATA', 'Invalid tickets configuration data');
}
function handleVerificationError(errors) {
    Logger.mainLogger.error('Ticket verification failed:', errors);
    return createApiError('INVALID_TICKET_SIGNATURES', 'Ticket verification failed', errors);
}
function validateTicketsArray(tickets) {
    if (!Array.isArray(tickets)) {
        Logger.mainLogger.error('Tickets data is not an array');
        return false;
    }
    return true;
}
function readAndValidateTickets() {
    const now = Date.now();
    // Check if we have valid cached tickets
    if (exports.ticketCache && (now - exports.ticketCache.lastRead) < CACHE_TTL) {
        return exports.ticketCache.tickets;
    }
    const jsonData = (0, fs_1.readFileSync)(ticketFilePath, 'utf8');
    const tickets = JSON.parse(jsonData);
    if (!validateTicketsArray(tickets)) {
        throw new Error('Invalid tickets format');
    }
    const verificationResult = (0, ticketVerification_1.verifyTickets)(tickets, verificationConfig);
    if (!verificationResult.isValid) {
        throw new Error(`Ticket verification failed: ${verificationResult.errors.map(e => e.message).join(', ')}`);
    }
    // Update cache
    exports.ticketCache = {
        tickets,
        lastRead: now
    };
    return tickets;
}
function initializeTickets() {
    try {
        readAndValidateTickets();
    }
    catch (err) {
        throw err; // This will prevent server from starting if tickets are invalid
    }
}
exports.initializeTickets = initializeTickets;
const ticketsRouter = function (fastify, opts, done) {
    // Add initialization before route handlers
    try {
        initializeTickets();
    }
    catch (err) {
        done(err);
        return;
    }
    // GET / - Get all tickets
    fastify.get('/', (_request, reply) => {
        try {
            const tickets = readAndValidateTickets();
            return reply.send(tickets);
        }
        catch (err) {
            if (err instanceof Error) {
                if (err.message.includes('ENOENT')) {
                    const error = handleFileError(err);
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err instanceof SyntaxError) {
                    const error = handleJsonParseError(err);
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err.message === 'Invalid tickets format') {
                    const error = createApiError('INVALID_TICKETS_FORMAT', 'Invalid tickets configuration format');
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err.message === 'Ticket verification failed') {
                    const error = createApiError('INVALID_TICKET_SIGNATURES', 'Ticket verification failed');
                    return reply.code(error.statusCode).send(error.response);
                }
            }
            const error = createApiError('INTERNAL_SERVER_ERROR', 'Internal server error');
            return reply.code(error.statusCode).send(error.response);
        }
    });
    // GET /:type - Get tickets by type
    fastify.get('/:type', (request, reply) => {
        const { type } = request.params;
        if (!type || typeof type !== 'string') {
            const error = createApiError('INVALID_TICKET_TYPE', 'Invalid ticket type parameter');
            return reply.code(error.statusCode).send(error.response);
        }
        try {
            const tickets = readAndValidateTickets();
            const ticket = tickets.find((t) => t.type === type);
            if (!ticket) {
                const error = createApiError('TICKET_NOT_FOUND', `No ticket found with type: ${type}`);
                return reply.code(error.statusCode).send(error.response);
            }
            return reply.send(ticket);
        }
        catch (err) {
            if (err instanceof Error) {
                if (err.message.includes('ENOENT')) {
                    const error = handleFileError(err);
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err instanceof SyntaxError) {
                    const error = handleJsonParseError(err);
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err.message === 'Invalid tickets format') {
                    const error = createApiError('INVALID_TICKETS_FORMAT', 'Invalid tickets configuration format');
                    return reply.code(error.statusCode).send(error.response);
                }
                if (err.message === 'Ticket verification failed') {
                    const error = createApiError('INVALID_TICKET_SIGNATURES', 'Ticket verification failed');
                    return reply.code(error.statusCode).send(error.response);
                }
            }
            const error = createApiError('INTERNAL_SERVER_ERROR', 'Internal server error');
            return reply.code(error.statusCode).send(error.response);
        }
    });
    done();
};
exports.ticketsRouter = ticketsRouter;
exports.default = exports.ticketsRouter;
//# sourceMappingURL=tickets.js.map